/**
 * \file framework.h
 * \brief フレームワーク・インクルードファイル
 * \details
 * 原則としてこのファイルの中の宣言をユーザーが変更する必要はない。例外はデバッグマクロのコメントアウト制御である。
 *
 * このファイルの中には複数のデバッグマクロが用意されており、これらのコメントアウトを説いて宣言状態にすると、DSP UARTのシリアルポートから
 * 内部状態を見ることができる。以下にマクロを列挙する。これらはフレームワークの開発中に使用したものをそのまま残している。
 *
 * ユーザーが使うとすれば、DEBUG_DSPLOADか、DEBUG_COMMANDだろう。
 *
 * \li DEBUG_QUEUEDEPTH シリアルポートから、タスク間Queueの深さを表示させる
 * \li DEBUG_DSPLOAD シリアルポートからDSP負荷率を表示させる
 * \li DEBUG_COMMAND DSPに送られてくるコマンドの表示
 * \li DEBUG_COMMAND_API 復調器から使えるAPIのデバッグ用
 * \li DEBUG_BLOCK_FLOAT WFMモードでブロック浮動小数点の整数化がうまく出来ているかダンプする。
 *
 */

#ifndef _FRAMEWORK_H_
#define _FRAMEWORK_H_

/*
 *  RX_IFアプリケーションのヘッダファイル
 */

#include <t_services.h>
#include "demodulator.h"


/*
 * デバッグマクロ
 *
 * コメントアウトすると機能がディセーブルになる
*/
// #define DEBUG_QUEUEDEPTH    // シリアルポートから、タスク間Queueの深さを表示させる
// #define DEBUG_DSPLOAD       // シリアルポートからDSP負荷率を表示させる
// #define DEBUG_COMMAND           // DSPに送られてくるコマンドの表示
// #define DEBUG_COMMAND_API       // 復調器から使えるAPIのデバッグ用
// #define DEBUG_BLOCK_FLOAT       // WFMモードでブロック浮動小数点の整数化がうまく出来ているかダンプする。

/**
 * \brief RX_IF受信SPORTバッファのDMAサイズ
 * \details
 * RX_IF受信SPORTバッファのDMAサイズ。単位は[count]。バイト単位ではなく
 * 転送カウントが単位なので注意。
 * ワイドFMモードの場合、1サンプルが16カウントになる。
 * 非ワイドFMモードでは、1サンプルが4カウント。
 *
 * バッファサイズが32の場合、ワイドFMモードでは2サンプル(63uS)ごとに、
 * 非ワイドFMモードでは8サンプル(252uS)ごとに割り込みがかかる。
 *
 * なお、このバッファサイズは必ず4の倍数にすること。
 */
#define RXIF_BUFSIZE 32

/**
 * \brief AF送信SPORTバッファのDMAサイズ
 * \details
 * AF送信SPORTバッファのDMAサイズ。単位は[count]。バイト単位ではなく
 * 転送カウントが単位なので注意。
 * 1サンプルが2カウント。
 *
 * バッファサイズが8の場合、4サンプル(126uS)ごとに割り込みがかかる。
 *
 * なお、このバッファサイズは必ず2の倍数にすること。
 */
#define AF_BUFSIZE 8


/**
 * \brief TASK_RX_IF と TASK_AF の間のデータキューサイズ
 * \details
 * 2つのタスクの間のキューのサイズ[エントリ]を指定する。両タスクの
 * 実行の間にはWideFM/nonWideFM切り替えと、割り込み周期の差異に起因する
 * ジッターが生じするため、キューが必要になる。
 *
 * キューのサイズを決める要素は2つある。ひとつはキュー受信側
 * (TASK_AF)がデータを読み取る周期、もうひとつはキュー送信側
 * (TASK_RX_IF)がデータを読み取る周期である。いずれも単位はサンプルで、
 * このうち大きなほうがデータキューのサイズを決める。
 *
 * キューはオーバーフローとアンダーフローを避けるように作らなければならない。
 * アンダーフローはキューの受信側の割り込みが先行した場合に発生する。
 * これを避けるには、受信側が先行した場合に備えてダミーデータを
 * キューに入れておく必要がある。この数は、先で論じた周期の一番大きい物に
 * しておく。
 *
 * オーバーフローはキューの送信側の割り込みが先行した場合に発生する。
 * これを避けるには、送信側が先行した場合に備えて空き領域を
 * 確保する必要がある。この数は先で論じた周期の一番大きい物に
 * しておく。
 *
 * 結論として、キューには、最も長い割り込み周期[単位はサンプル]の倍の
 * 領域が必要である。
 *
 * なお、このソフトウェアは1サンプルあたり2[エントリ]をキューに格納するため、
 * これを勘案すると、最低でもキューの長さ[エントリ]は、割り込み周期[サンプル]の四倍になる。
 *
 *
 */
#define AF_QUESIZE          80      /* 送受信タスクの間のキューの長さ。8サンプルが最長割り込み周期と仮定してさらにマージンをもたせている */

#define WIDE_FM_OVERSAMPE   8       /* AF1サンプルあたりのWide FM サンプル数 */

#define WIDE_FM_FLAG        2       /* プライマリ・チャンネルのWIDE FM Modeを表すフラグのマスク */

#define PARAMDATA_NUM 16


/*
 *  各タスクの優先度の定義
 */

#define TASK_PRIORITY_AF       5  /* AF DMAタスクの優先度。送信なので優先度高 */
#define TASK_PRIORITY_RX_IF   10  /* RX-IF DMA処理タスクの優先度。復調も行う */
#define LOGTASK_PRIORITY      12  /* デフォルトの初期優先度を上書きする */
#define TASK_PRIORITY_MONITOR 15  /* モニター・タスクの優先度。優先度低 */

/*
 *  ターゲット依存の定義（CPU例外ハンドラの起動方法など）
 */


#define STACK_SIZE_RX_IF    1024    /* タスクのスタックサイズ */
#define STACK_SIZE_AF       1024    /* タスクのスタックサイズ */
#define STACK_SIZE_MONITOR  1024    /* タスクのスタックサイズ */
#define TASK_PORTID         1       /* 文字入力するシリアルポートID */



/*
 *  関数のプロトタイプ宣言
 */
#ifndef _MACRO_ONLY

/**
 * \brief フレームワークがラジオとのやりとりに使う変数。
 * \details
 * フレームワークはFPGAから送られてきたデータをこの変数の中に格納する。復調アルゴリズムは \ref getInfo を通してこれらの値にアクセスする。
 */

    // ラジオアルゴリズムとのやりとりに使う変数
extern struct RADIO {
    unsigned short flags;
    unsigned short smeter;
    unsigned short comdata[PARAMDATA_NUM];
} radio;

extern struct RADIO radio;


extern void rx_if_task(VP_INT exinf);   /* SPORT0 RX DMAバッファの内容を復調するタスク */
extern void af_task(VP_INT exinf);      /* 復調済みAF信号をSPORT0 TX DMAバッファにコピーするタスク */
extern void monitor_task(VP_INT exinf);   /* SPORT0 TX DMAバッファにAFデータを書き込むタスク */
extern void sport0_rx_int_handler( void );  /* SPORT0 RX DMA割り込みのハンドラ */
extern void sport0_tx_int_handler( void );  /* SPORT0 TX DMA割り込みのハンドラ */
void init_peripherals(VP_INT p);        /* ペリフェラルの初期化関数 */


#endif /* _MACRO_ONLY */

#endif /* _FRAMEWORK_H_ */
